<?php

/**
 * @noinspection PhpUnused
 * @noinspection PhpUnusedParameterInspection
 */

namespace TelegramApi;

use TelegramApi\Types\ForceReply;
use TelegramApi\Types\InputFile;
use TelegramApi\Types\Message;
use TelegramApi\Types\ReplyKeyboardHide;
use TelegramApi\Types\ReplyKeyboardMarkup;
use TelegramApi\Types\Type;
use TelegramApi\Types\Update;
use TelegramApi\Types\User;
use TelegramApi\Types\UserProfilePhotos;

trait API
{
	abstract public function sendRequest(string $method, array $args): mixed;


	/**
	 * Use this method to receive incoming updates using long polling (wiki). An Array of Update objects is returned.
	 * @param Type|null $parameters Description
	 * @param int|null $offset Identifier of the first update to be returned. Must be greater by one than the highest among the identifiers of previously received updates. By default, updates starting with the earliest unconfirmed update are returned. An update is considered confirmed as soon as getUpdates is called with an offset higher than its update_id.
	 * @param int|null $limit Limits the number of updates to be retrieved. Values between 1—100 are accepted. Defaults to 100
	 * @param int|null $timeout Timeout in seconds for long polling. Defaults to 0, i.e. usual short polling
	 * @return Array<Update>
	 */
	public function getUpdates(
		?Type $parameters = null,
		?int $offset = null,
		?int $limit = 100,
		?int $timeout = null,
	): array
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to specify a url and receive incoming updates via an outgoing webhook. Whenever there is an update for the bot, we will send an HTTPS POST request to the specified url, containing a JSON-serialized Update. In case of an unsuccessful request, we will give up after a reasonable amount of attempts.
	 * If you'd like to make sure that the Webhook request comes from Telegram, we recommend using a secret path in the URL, e.g. www.example.com/<token>. Since nobody else knows your bot‘s token, you can be pretty sure it’s us.
	 * @param Type|null $parameters Description
	 * @param string|null $url HTTPS url to send updates to. Use an empty string to remove webhook integration
	 */
	public function setWebhook(?Type $parameters = null, ?string $url = null)
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * A simple method for testing your bot's auth token. Requires no parameters. Returns basic information about the bot in form of a User object.
	 * @return User
	 */
	public function getMe(): User
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send text messages. On success, the sent Message is returned.
	 * @param int $chatId Unique identifier for the message recipient — User or GroupChat id
	 * @param string $text Text of the message to be sent
	 * @param Type|null $parameters Description
	 * @param bool|null $disableWebPagePreview Disables link previews for links in this message
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for a custom reply keyboard, instructions to hide keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendMessage(
		int $chatId,
		string $text,
		?Type $parameters = null,
		?bool $disableWebPagePreview = null,
		?int $replyToMessageId = null,
		ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to forward messages of any kind. On success, the sent Message is returned.
	 * @param int $chatId Unique identifier for the message recipient — User or GroupChat id
	 * @param int $fromChatId Unique identifier for the chat where the original message was sent — User or GroupChat id
	 * @param int $messageId Unique message identifier
	 * @param Type|null $parameters Description
	 * @return Message
	 */
	public function forwardMessage(int $chatId, int $fromChatId, int $messageId, ?Type $parameters = null): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send photos. On success, the sent Message is returned.
	 * @param int $chatId Unique identifier for the message recipient — User or GroupChat id
	 * @param InputFile|string $photo Photo to send. You can either pass a file_id as String to resend a photo that is already on the Telegram servers, or upload a new photo using multipart/form-data.
	 * @param Type|null $parameters Description
	 * @param string|null $caption Photo caption (may also be used when resending photos by file_id).
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for a custom reply keyboard, instructions to hide keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendPhoto(
		int $chatId,
		InputFile|string $photo,
		?Type $parameters = null,
		?string $caption = null,
		?int $replyToMessageId = null,
		ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send audio files, if you want Telegram clients to display the file as a playable voice message. For this to work, your audio must be in an .ogg file encoded with OPUS (other formats may be sent as Document). On success, the sent Message is returned. Bots can currently send audio files of up to 50 MB in size, this limit may be changed in the future.
	 * @param int $chatId Unique identifier for the message recipient — User or GroupChat id
	 * @param InputFile|string $audio Audio file to send. You can either pass a file_id as String to resend an audio that is already on the Telegram servers, or upload a new audio file using multipart/form-data.
	 * @param Type|null $parameters Description
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for a custom reply keyboard, instructions to hide keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendAudio(
		int $chatId,
		InputFile|string $audio,
		?Type $parameters = null,
		?int $replyToMessageId = null,
		ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send general files. On success, the sent Message is returned. Bots can currently send files of any type of up to 50 MB in size, this limit may be changed in the future.
	 * @param int $chatId Unique identifier for the message recipient — User or GroupChat id
	 * @param InputFile|string $document File to send. You can either pass a file_id as String to resend a file that is already on the Telegram servers, or upload a new file using multipart/form-data.
	 * @param Type|null $parameters Description
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for a custom reply keyboard, instructions to hide keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendDocument(
		int $chatId,
		InputFile|string $document,
		?Type $parameters = null,
		?int $replyToMessageId = null,
		ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send .webp stickers. On success, the sent Message is returned.
	 * @param int $chatId Unique identifier for the message recipient — User or GroupChat id
	 * @param InputFile|string $sticker Sticker to send. You can either pass a file_id as String to resend a sticker that is already on the Telegram servers, or upload a new sticker using multipart/form-data.
	 * @param Type|null $parameters Description
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for a custom reply keyboard, instructions to hide keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendSticker(
		int $chatId,
		InputFile|string $sticker,
		?Type $parameters = null,
		?int $replyToMessageId = null,
		ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send video files, Telegram clients support mp4 videos (other formats may be sent as Document). On success, the sent Message is returned. Bots can currently send video files of up to 50 MB in size, this limit may be changed in the future.
	 * @param int $chatId Unique identifier for the message recipient — User or GroupChat id
	 * @param InputFile|string $video Video to send. You can either pass a file_id as String to resend a video that is already on the Telegram servers, or upload a new video file using multipart/form-data.
	 * @param Type|null $parameters Description
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for a custom reply keyboard, instructions to hide keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendVideo(
		int $chatId,
		InputFile|string $video,
		?Type $parameters = null,
		?int $replyToMessageId = null,
		ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send point on the map. On success, the sent Message is returned.
	 * @param int $chatId Unique identifier for the message recipient — User or GroupChat id
	 * @param float $latitude Latitude of location
	 * @param float $longitude Longitude of location
	 * @param Type|null $parameters Description
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for a custom reply keyboard, instructions to hide keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendLocation(
		int $chatId,
		float $latitude,
		float $longitude,
		?Type $parameters = null,
		?int $replyToMessageId = null,
		ReplyKeyboardMarkup|ReplyKeyboardHide|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method when you need to tell the user that something is happening on the bot's side. The status is set for 5 seconds or less (when a message arrives from your bot, Telegram clients clear its typing status).
	 * We only recommend using this method when a response from the bot will take a noticeable amount of time to arrive.
	 * @param int $chatId Unique identifier for the message recipient — User or GroupChat id
	 * @param string $action Type of action to broadcast. Choose one, depending on what the user is about to receive: typing for text messages, upload_photo for photos, record_video or upload_video for videos, record_audio or upload_audio for audio files, upload_document for general files, find_location for location data.
	 * @param Type|null $parameters Description
	 */
	public function sendChatAction(int $chatId, string $action, ?Type $parameters = null)
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to get a list of profile pictures for a user. Returns a UserProfilePhotos object.
	 * @param int $userId Unique identifier of the target user
	 * @param Type|null $parameters Description
	 * @param int|null $offset Sequential number of the first photo to be returned. By default, all photos are returned.
	 * @param int|null $limit Limits the number of photos to be retrieved. Values between 1—100 are accepted. Defaults to 100.
	 * @return UserProfilePhotos
	 */
	public function getUserProfilePhotos(
		int $userId,
		?Type $parameters = null,
		?int $offset = null,
		?int $limit = 100,
	): UserProfilePhotos
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}
}
